from .file_base import BaseFile as BaseFile
from .file_generator import GeneratorFileContract as GeneratorFileContract
from .file_generator import GeneratorFileExport as GeneratorFileExport
from .generator import GeneratorContract as GeneratorContract
from .renderer import RendererContract as RendererContract



from typing import List, Optional

from fabricius.contracts import GeneratorContract
from fabricius.contracts.file_generator import CommitResult
from fabricius.errors import AlreadyCommittedError
from fabricius.file import FileGenerator
from fabricius.reporter import Reporter


class Generator(GeneratorContract):
    reporter = Reporter()

    def __init__(self) -> None:
        self.files = []

    def add_file(self, name: str, extension: Optional[str] = None):
        file = FileGenerator(name, extension)
        self.files.append(file)
        return file

    def execute(self, *, allow_overwrite: bool = False):
        result: List[CommitResult] = []
        for file in self.files:
            try:
                file_result = file.commit(overwrite=allow_overwrite)
                path = str(file_result["path"].absolute())
                if file_result["state"] == "persisted":
                    self.reporter.success(f"{path} ({file.name})", title="CREATED")
                elif file_result["state"] == "pending":
                    self.reporter.skip(f"{path} ({file.name})")
                result.append(file_result)
            except AlreadyCommittedError:
                self.reporter.skip(f"{file.name} - Already committed")
            except Exception:
                self.reporter.fail(file.name)
                self.reporter.console.print_exception(extra_lines=0)

        return result



from importlib.metadata import version as __package_version

from .const import FILE_STATE as FILE_STATE
from .const import Data as Data
from .errors import AlreadyCommittedError as AlreadyCommittedError
from .errors import FabriciusError as FabriciusError
from .errors import NoContentError as NoContentError
from .errors import NoDestinationError as NoDestinationError
from .file import FileGenerator as FileGenerator
from .generator import Generator as Generator
from .renderer import ChevronRenderer as ChevronRenderer
from .renderer import PythonFormatRenderer as PythonFormatRenderer
from .renderer import StringTemplateRenderer as StringTemplateRenderer
from .utils import camel_case as camel_case
from .utils import capital_case as capital_case
from .utils import dash_case as dash_case
from .utils import pascal_case as pascal_case
from .utils import sentence_case as sentence_case
from .utils import snake_case as snake_case

__version__ = __package_version("fabricius")



from abc import ABC

from fabricius.const import FILE_STATE


class BaseFile(ABC):
    """
    Base class for files, not much of a contract, but contain 2 important properties.

    .. property:: name

       The name of the file.

       :type: str

    .. property:: state

       The state of the file.

       :type: :py:data:`.FILE_STATE`
    """

    name: str
    state: FILE_STATE



from typing import Any, Dict, Literal, TypeAlias

Data: TypeAlias = Dict[str, Any]
"""
Represent the data passed to a generator/file, under a form of dictionary.
"""

FILE_STATE = Literal["pending", "persisted", "deleted"]
"""
Define the state of a file.

The file's state can be one of:

* pending
* persisted
* deleted
"""



from abc import ABC, abstractmethod
from typing import List, Optional

from fabricius.contracts.file_generator import CommitResult

from .file_generator import GeneratorFileContract


class GeneratorContract(ABC):
    files: List[GeneratorFileContract]

    @abstractmethod
    def add_file(self, name: str, extension: Optional[str] = None) -> GeneratorFileContract:
        """
        Add a file to the generator.

        Parameters
        ----------
        name : str
            The name of the file
        extension: Optional, str
            The extension of the file, can be optional.
            If none, no extension will be added.

        Returns
        -------
        GeneratorFileContract :
            The generated file. You then have to set file's options.
        """
        raise NotImplementedError()

    @abstractmethod
    def execute(self) -> List[CommitResult]:
        """
        Execute generator's tasks.

        Returns
        -------
        """
        raise NotImplementedError()



from typing import Optional

import rich
from rich.style import Style
from rich.text import Text


class Reporter:
    def __init__(self) -> None:
        self.console: rich.console.Console = rich.get_console()

    def success(self, message: str, *, title: Optional[str] = None):
        text = self.stylize_text(title or "SUCCESS", "green", message)
        self.console.print(text)

    def fail(self, message: str, *, title: Optional[str] = None):
        text = self.stylize_text(title or "FAIL", "red", message)
        self.console.print(text)

    def skip(self, message: str, *, title: Optional[str] = None):
        text = self.stylize_text(title or "SKIPPED", "bright_cyan", message)
        self.console.print(text)

    @staticmethod
    def stylize_text(status: str, color: str, message: str) -> Text:
        text = Text(f" {status} ", style=Style(bgcolor=color))
        text.append(f" {message}", style=Style(bgcolor="black", color=color))
        return text


if __name__ == "__main__":
    r = Reporter()
    r.success("Success!")
    r.fail("Fail!")
    r.skip("Skipped!")



from typing import Optional


class FabriciusError(Exception):
    """
    An error was raised inside Fabricius.
    """

    def __init__(self, error: Optional[object] = None) -> None:
        """
        An error was raised inside Fabricius.
        """
        super().__init__(error or "Error inside Fabricius. No specific error raised.")


class NoContentError(FabriciusError):
    """
    The file does not have any content.
    """

    def __init__(self, file_name: str) -> None:
        """
        The file does not have any content.
        """
        super().__init__(f"File '{file_name}' does not have content from a template.")


class NoDestinationError(FabriciusError):
    """
    The file does not know where to go.
    """

    def __init__(self, file_name: str) -> None:
        """
        The file does not know where to go.
        """
        super().__init__(f"File '{file_name}' does not have a destination set.")


class AlreadyCommittedError(FabriciusError):
    """
    The file has already been committed/persisted.
    """

    def __init__(self, file_name: str) -> None:
        """
        The file has already been committed/persisted.
        """
        super().__init__(f"File '{file_name}' has already been committed.")



import inflection


def camel_case(text: str) -> str:
    """
    Return the text formatted in camel case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> camel_case(my_text)
       'someText'
    """
    return inflection.camelize(text, False)


def snake_case(text: str) -> str:
    """
    Return the text formatted in snake case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> snake_case(my_text)
       'some_text'
    """
    return inflection.underscore(text)


def dash_case(text: str) -> str:
    """
    Return the text formatted in dash case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> dash_case(my_text)
       'some-text'
    """
    return inflection.dasherize(text)


def pascal_case(text: str) -> str:
    """
    Return the text formatted in pascal case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> pascal_case(my_text)
       'SomeText'
    """
    return inflection.camelize(text, True)


def capital_case(text: str) -> str:
    """
    Return the text formatted in capital case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> capital_case(my_text)
       'Some Text'
    """
    return inflection.titleize(text)


def sentence_case(text: str) -> str:
    """
    Return the text formatted in sentence case

    Parameters
    ----------
    text: str
        The text you want to format.

    Returns
    -------
    str:
        The formatted text.

    Example
    -------
    .. code-block:: python

       >>> my_text = "Some text"
       >>> sentence_case(my_text)
       'Some text'
    """
    has_ending_id: bool = False
    if text.endswith("_id"):
        has_ending_id = True

    result = inflection.humanize(text)
    if has_ending_id:
        result += " ID"
    return result



from abc import ABC, abstractmethod

from fabricius.const import Data


class RendererContract(ABC):
    """
    A class with purpose to render a string with data given on initialization.

    .. property:: data

       A dictionary that contains data that has been passed to the renderer.
       Useful to render in a file.

       :type: :py:data:`.Data`
    """

    data: Data

    def __init__(self, data: Data) -> None:
        self.data = data

    @abstractmethod
    def render(self, content: str) -> str:
        """
        Render a string.

        Example:

        .. code-block:: python

           class MyRenderer(RendererContract):

               def render(self, content: str) -> str:
                   return content.format(self.data)

           content = MyRenderer({"thing": "world"}).render('Please render the {thing}!')
           print(content)
           # Please render the world!
        """
        raise NotImplementedError()



import pathlib
from abc import abstractmethod
from typing import Optional, Type, TypedDict

from typing_extensions import Self

from fabricius.const import FILE_STATE, Data

from .file_base import BaseFile
from .renderer import RendererContract


class CommitResult(TypedDict):
    """
    Return the result of a file commit.

    .. property:: state

       The state of the file.

       :type: :py:const:`FILE_STATE <fabricius.const.FILE_STATE>`

    .. property:: with_data

       The data that has been passed to the file when rendering.
       None if the commit failed.

       :type: Optional, :py:const:`Data <fabricius.const.Data>`

    .. property:: content

       The final content of the file once created.

       :type: :py:class:`str`

    .. property:: path

       The path of the file.
       None if the commit failed.

       :type: Optional, :py:class:`pathlib.Path`

    """

    state: FILE_STATE
    with_data: Data
    template_content: str
    content: str
    path: pathlib.Path


class GeneratorFileExport(TypedDict):
    """
    A TypedDict object. Used for :py:func:`GeneratorFileContract.to_dict`.

    .. property:: name

       The name of the file.

       :type: :py:class:`str`

    .. property:: state

       The actual state of the file.

       :type: :py:data:`.FILE_STATE`

    .. property:: template_content

       The content of the template file, if any.

       :type: Optional, :py:class:`str`

    .. property:: destination

       The selected destination for the file, if any.

       :type: Optional, :py:class:`pathlib.Path`

    .. property:: renderer

       The renderer that will render the result.

       :type: Type[:py:class:`.RendererContract`]
    """

    name: str
    state: FILE_STATE

    template_content: Optional[str]
    destination: Optional[pathlib.Path]

    renderer: Type[RendererContract]
    data: Data


class GeneratorFileContract(BaseFile):
    content: Optional[str]
    destination: Optional[pathlib.Path]

    renderer: Type[RendererContract]
    data: Data

    @abstractmethod
    def __init__(self, name: str, extension: Optional[str]) -> None:
        """
        Create a file's generator.

        Parameters
        ----------
        name : str
            The name of the file.
        extension : str
            The extension of the file, without dot, same as ``name="<name>.<extension>"``
        """
        raise NotImplementedError()

    @abstractmethod
    def from_file(self, path: str | pathlib.Path) -> Self:
        """
        Read the content from a file template.

        Parameters
        ----------
        path : str or pathlib.Path
            The path of the file template.
        """
        raise NotImplementedError()

    @abstractmethod
    def from_content(self, content: str) -> Self:
        """
        Read the content from a string.

        Parameters
        ----------
        content : str
            The template you want to format.
        """
        raise NotImplementedError()

    @abstractmethod
    def to_directory(self, directory: str | pathlib.Path, *, recursive: bool = False) -> Self:
        """
        Set the directory where the file will be saved.

        Raises
        ------
        FileNotFoundError :
            The given directory does not exist. (And recursive is set to False)

        Parameters
        ----------
        directory : str or pathlib.Path
            Where the file will be stored. Does not include the file's name.
        recursive : bool
            Create the parents folder if not existing. Default to True.
        """
        raise NotImplementedError()

    @abstractmethod
    def use_mustache(self) -> Self:
        """
        Use chevron (Mustache) to render the template.
        """
        raise NotImplementedError()

    @abstractmethod
    def use_string_template(self) -> Self:
        """
        Use string.Template to render the template.
        """
        raise NotImplementedError()

    @abstractmethod
    def with_renderer(self, renderer: Type[RendererContract]) -> Self:
        """
        Use a custom renderer to render the template.

        Parameters
        ----------
        renderer : Type[RendererContract]
            A renderer that implements :py:class:`.RendererContract`.
            It must be not initialized.
        """

    @abstractmethod
    def with_data(self, data: Data, *, overwrite: bool = False) -> Self:
        """
        Add data to pass to the template.

        Parameters
        ----------
        data : :py:data:`.Data`
            The data you want to pass to the template.
        overwrite : bool
            If the data that already exists should be deleted. If False, the new data will be
            added on top of the already existing data.
        """
        raise NotImplementedError()

    @abstractmethod
    def generate(self) -> str:
        """
        Generate the file's content.

        Raises
        ------
        NoContentError :
            If no content to the file were added.

        Returns
        -------
        str :
            The final content of the file.
        """
        raise NotImplementedError()

    @abstractmethod
    def commit(self, *, overwrite: bool = False) -> CommitResult:
        """
        Save the file using this class.

        Parameters
        ----------
        overwrite : bool
            If a file exist at the given path, shall the overwrite parameter say if the file should be overwritten or net.

        Raises
        ------
        NoContentError :
            If no content to the file were added.
        NoDestinationError :
            If no destination/path were designated.

        Returns
        -------
        CommitResult :
            A dict with information about the created file.
        """
        raise NotImplementedError()

    @abstractmethod
    def to_dict(self) -> GeneratorFileExport:
        """
        Export the class to a typed dict.

        Returns
        -------
        GeneratorFileExport :
            The exported dict.
        """
        raise NotImplementedError()



import pathlib
from typing import Optional, Type

from typing_extensions import Self

from fabricius.const import Data
from fabricius.contracts import GeneratorFileContract, RendererContract
from fabricius.contracts.file_generator import CommitResult, GeneratorFileExport
from fabricius.errors import NoContentError, NoDestinationError
from fabricius.renderer import (
    ChevronRenderer,
    PythonFormatRenderer,
    StringTemplateRenderer,
)


class FileGenerator(GeneratorFileContract):
    def __init__(self, name: str, extension: Optional[str] = None):
        self.name = f"{name}.{extension}" if extension else name
        self.state = "pending"
        self.content = None
        self.destination = None

        self.renderer = PythonFormatRenderer
        self.data = {}

    def from_file(self, path: str | pathlib.Path) -> Self:
        path = pathlib.Path(path) if isinstance(path, str) else path
        with open(path.absolute(), "r") as file_content:
            self.content = file_content.read()
        return self

    def from_content(self, content: str) -> Self:
        self.content = content
        return self

    def to_directory(self, directory: str | pathlib.Path, *, recursive: bool = True) -> Self:
        path = pathlib.Path(directory).absolute()
        if not path.exists():
            path.mkdir(parents=recursive)
        if not path.is_dir():
            raise NotADirectoryError(f"{path} is not a directory.")
        self.destination = path
        return self

    def use_mustache(self) -> Self:
        self.renderer = ChevronRenderer
        return self

    def use_string_template(self) -> Self:
        self.renderer = StringTemplateRenderer
        return self

    def with_renderer(self, renderer: Type[RendererContract]) -> Self:
        self.renderer = renderer
        return self

    def with_data(self, data: Data, *, overwrite: bool = False) -> Self:
        if overwrite:
            self.data = {}
        self.data.update(data)
        return self

    def generate(self) -> str:
        if not self.content:
            raise NoContentError(self.name)

        return self.renderer(self.data).render(self.content)

    def commit(self, *, overwrite: bool = False) -> CommitResult:
        # Sourcery propose to replace the try-except for contextlib.suppress
        # It is skipped because the state would always be persisted that way, which is wrong.
        # sourcery skip: use-contextlib-suppress
        if not self.destination:
            raise NoDestinationError(self.name)

        final_content = self.generate()
        destination = self.destination.joinpath(self.name)

        if destination.exists() and not overwrite:
            return self._build_commit_result(final_content)

        try:
            destination.write_text(final_content)
            self.state = "persisted"
        except NotADirectoryError:
            pass

        return self._build_commit_result(final_content)

    def to_dict(self) -> GeneratorFileExport:
        return GeneratorFileExport(
            name=self.name,
            state=self.state,
            template_content=self.content,
            destination=self.destination,
            renderer=self.renderer,
            data=self.data,
        )

    def _build_commit_result(self, final_content: str) -> CommitResult:
        """
        Build a commit result.

        It is not recommended to use it in your own codebase.

        Returns
        -------
        CommitResult :
            The commit's result.
        """
        if not self.content:
            raise NoContentError(self.name)
        if not self.destination:
            raise NoDestinationError(self.name)

        return CommitResult(
            state=self.state,
            with_data=self.data,
            template_content=self.content,
            content=final_content,
            path=self.destination.joinpath(self.name),
        )



import string
from collections import UserDict
from typing import Any

import chevron

from fabricius.contracts import RendererContract


class DictAllowMiss(UserDict[str, Any]):
    """
    A subclass of Dict to return an empty string in case of missing key, instead of raising an
    error, so that it can play nice with renderer.

    :meta private:
    """

    def __missing__(self, _: str):
        return ""


class PythonFormatRenderer(RendererContract):
    def render(self, content: str) -> str:
        return content.format_map(DictAllowMiss(self.data))


class ChevronRenderer(RendererContract):
    def render(self, content: str) -> str:
        return chevron.render(content, self.data)


class StringTemplateRenderer(RendererContract):
    def render(self, content: str) -> str:
        return string.Template(content).safe_substitute(DictAllowMiss(self.data))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['fabricius', 'fabricius.contracts']

package_data = \
{'': ['*']}

install_requires = \
['chevron>=0.14.0,<0.15.0',
 'inflection>=0.5.1,<0.6.0',
 'rich>=12.4.4,<13.0.0',
 'typing-extensions>=4.2.0,<5.0.0']

extras_require = \
{'docs': ['Sphinx>=5.0.2,<6.0.0',
          'furo>=2022.6.21,<2023.0.0',
          'sphinx-autobuild>=2021.3.14,<2022.0.0']}

setup_kwargs = {
    'name': 'fabricius',
    'version': '0.0.3',
    'description': 'Fabricius: The supportive templating engine for Python!',
    'long_description': "# Fabricius\n\nFabricius - A Python 3.10 Project Template engine with superpowers!\n\n> :warning: Fabricius is a work in progress! Please, play with it with a grain of salt; expect bugs, crashes, non-documented portion of the application & more unexpected behavior.\n\nDocumentation: <https://fabricius.readthedocs.io>\n\n> :warning: Fabricius still does not comes with it's CLI tool! It is a work in progress!\n",
    'author': 'Predeactor',
    'author_email': 'pro.julien.mauroy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Predeactor/Fabricius',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
